import socket
import struct


def ip2int(ip_address):
    if not isinstance(ip_address, str):
        raise TypeError("Value %s is not a type of string" % ip_address)
    return struct.unpack("!I", socket.inet_aton(ip_address))[0]


def int2ip(integer):
    if type(integer).__name__ not in ("int", "long"):
        # hack: since in the python3, long is not supported
        raise TypeError("Value %s is not a number" % integer)
    return socket.inet_ntoa(struct.pack("!I", integer))



from unittest import TestCase

from scapy.layers.l2 import Ether
from scapy_helper.main import show_diff


class TestShowDiff(TestCase):
    def setUp(self):
        self.ether_1 = Ether()

    def test_show_diff(self):
        ether_2 = Ether(dst="00:11:00:00:00:00")
        show_diff(self.ether_1, ether_2)

    def test_show_diff__w_additional_str(self):
        ether_2 = Ether(dst="00:11:00:00:00:00") / "additional str"
        show_diff(self.ether_1, ether_2)

    def test_show_diff_between_ether_obj_v_str(self):
        ether_2 = "ff ff fc ff ff fa 00 11 11 11 11 11 90 00 11 11 00 22"
        show_diff(self.ether_1, ether_2)

    def test_show_diff_with_indexes(self):
        ether_2 = "ff ff fc ff ff fa 00 11 11 11 11 11 90 00 11 11 00 22"
        show_diff(self.ether_1, ether_2, index=True)

    def test_show_diff_with_indexes_and_custom_empty_char(self):
        ether_2 = "ff ff fc ff ff fa 00 11 11 11 11 11 90 00 11 11 00 22"
        show_diff(self.ether_1, ether_2, index=True, empty_char="++")

    def test_show_diff_with_indexes_and_custom_too_long_empty_char(self):
        ether_2 = "ff ff fc ff ff fa 00 11 11 11 11 11 90 00 11 11 00 22"
        show_diff(self.ether_1, ether_2, index=True, empty_char="+Only Plus Char should be used")

    def test_show_diff_with_indexes_and_custom_char_none(self):
        ether_2 = "ff ff fc ff ff fa 00 11 11 11 11 11 90 00 11 11 00 22"
        show_diff(self.ether_1, ether_2, index=True, empty_char=None)

    def test_show_diff_with_indexes_proper_len_of_last_position(self):
        ether_2 = "ff ff fc ff ff fa 00 11 11 11 11 11 90 00"
        show_diff(self.ether_1, ether_2, index=True, empty_char=None)



from scapy_helper import hex_equal, show_diff
from scapy_helper.main import show_diff_full, diff, get_hex


def failure(first, second, message):
    raise AssertionError(get_hex(first), get_hex(second), message)


class PacketAssert:
    @staticmethod
    def assertHexEqual(first, second, message=None):
        if not hex_equal(first, second, show_inequalities=False):
            show_diff_full(first, second)
            failure(first, second, message)

    @staticmethod
    def assertHexNotEqual(first, second, message=None):
        if hex_equal(first, second, show_inequalities=False):
            show_diff(first, second)
            failure(first, second, message)

    @staticmethod
    def assertBytesEqual(first, second, message=None):
        if not isinstance(first, bytes):
            first = bytes(first)
        if not isinstance(second, bytes):
            second = bytes(second)

        if first != second:
            failure(first, second, message)

    @staticmethod
    def assertBytesNotEqual(first, second, message=None):
        if not isinstance(first, bytes):
            first = bytes(first)
        if not isinstance(second, bytes):
            second = bytes(second)

        if first == second:
            failure(first, second, message)

    @staticmethod
    def assertHexDifferentAt(first, second, positions, message):
        _, _, differences_at_position = diff(first, second)
        if len(differences_at_position) == 0:
            print("Hexs are equal")
            failure(first, second, message)

        if isinstance(positions, int):
            if len(differences_at_position) != 1 or (positions not in differences_at_position):
                show_diff(first, second, index=True)
                failure(first, second, message)
        elif isinstance(positions, (list, tuple)):
            if not set(positions).issubset(set(differences_at_position)) or set(positions) < set(
                    differences_at_position):
                show_diff(first, second, index=True)
                failure(first, second, message)



def deprecated(func, *args, **kwargs):
    """
    Signal a user deprecation of specific functions
    :param func:
    :param args:
    :param kwargs:
    :return:
    """
    def wrapper():
        print("WARN:: Deprecated %s" % func.__name__)
        func(*args, **kwargs)
    return wrapper



from unittest import TestCase

from scapy_helper.helpers.mac import int2mac, mac2int


class TestMac(TestCase):
    def setUp(self):
        self.int_mac = 73596036829
        self.str_mac = "00:11:22:AA:66:DD"

    def test_int_to_mac(self):
        self.assertEqual(
            int2mac(self.int_mac),
            self.str_mac.lower(),
            "Values should be equal"
        )

    def test_int_to_mac_upper(self):
        self.assertEqual(
            int2mac(self.int_mac, upper=True),
            self.str_mac,
            "Values should be equal"
        )

    def test_mac_to_int(self):
        self.assertEqual(
            mac2int(self.str_mac),
            self.int_mac,
            "Values should be equal"
        )



from scapy_helper.main import get_hex, show_diff, show_hex, table, hex_equal
from scapy_helper.hexdump import hexdump
from scapy_helper.chexdump import chexdump
from scapy_helper.compare import Compare

from scapy_helper.helpers.mac import mac2int, int2mac

from scapy_helper.test_case_extensions.packet_assert import PacketAssert



import unittest

from scapy.layers.l2 import Ether

from scapy_helper.chexdump import chexdump


class TestcHexdump(unittest.TestCase):
    def setUp(self):
        self.packet = Ether(dst="ff:ff:ff:ff:ff:ff",
                            src="00:00:00:00:00:00")

    def test_chexdump(self):
        # TODO Add a verification of the output
        packet = "\x00\x01".encode()
        chexdump(packet)

    def test_chexdump_dump_true(self):
        expected_result = '0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x00'
        result = chexdump(self.packet, dump=True)

        self.assertIsInstance(result, str, "Dump should be a string")
        self.assertEqual(expected_result, result, "Dump of the hex should be the same")

    def test_chexdump_dump_true_as_list(self):
        expected_result = ['0xff', '0xff', '0xff', '0xff', '0xff', '0xff',
                           '0x00', '0x00', '0x00', '0x00', '0x00', '0x00',
                           '0x90', '0x00']
        result = chexdump(self.packet, dump=True, to_list=True)

        self.assertIsInstance(result, list, "Dump should be a string")
        self.assertEqual(result, expected_result, "Dump of the hex should be the same")



from scapy_helper.main import _prepare, get_hex


def hexdump(packet, dump=False, to_list=False):
    def to_number(number):
        return number if isinstance(number, int) else ord(number)

    def to_char(string):
        j = to_number(string)
        if (j < 32) or (j >= 127):
            return "."
        else:
            return chr(j)

    def split(obj, num):
        return [obj[start:start + num]
                for start in range(0, len(obj), num)
                if obj[start:start + num]]

    def __process_hexdump(ppacket):
        row = []
        for i, line in enumerate(split(ppacket, 16)):
            console_char = [to_char(int(x, 16)) for x in line]
            if len(line) < 16:
                line += ["  " for _ in range(16 - len(line))]

            row.append("%03x0   %s   %s" %
                       (i, ' '.join(line), ''.join(console_char)))
        return row

    def __alternative_process_hexdump(ppacket):
        row = []
        for i, line in enumerate(split(ppacket, 16)):
            console_char = [to_char(x) for x in line]
            if len(line) < 16:
                line += ["  " for _ in range(16 - len(line))]

            row.append("%03x0   %s   %s" %
                       (i, ' '.join(line), ''.join(console_char)))
        return row

    try:
        processed_packet = _prepare(packet)
        if not processed_packet:
            return
        rows = __process_hexdump(processed_packet)
    except ValueError:
        # It's not a ideal workaround for detecting a proper
        # format of input, but works on Py2 & Py3
        processed_packet = get_hex(packet)
        if not processed_packet:
            return
        rows = __alternative_process_hexdump(processed_packet)
    if dump:
        if to_list:
            return rows
        return "\n".join(rows).rstrip("\n")
    print("\n".join(rows).rstrip("\n"))



import sys

from scapy_helper.helpers.depracated import deprecated


def diff(*args, **kwargs):
    """
    Return a diff between two hex list
    :param args:
    :param kwargs:
    :return:
    """
    skip_if_same = True
    if kwargs.get("skip_if_same", False):
        skip_if_same = kwargs["skip_if_same"]
    if len(args) != 2:
        raise NotImplementedError("Only comparison of the two list are supported")

    result_list = ([], [])
    diff_indexes = []
    diff_list = []

    for a in args:
        diff_list.append(_prepare(a))
    _fill_empty_elements(*diff_list)

    for e, _ in enumerate(diff_list[0]):
        if diff_list[0][e].lower() != diff_list[1][e].lower():
            for i in range(2):
                result_list[i].append(diff_list[i][e])
            diff_indexes.append(e)
            continue
        if skip_if_same:
            for i in range(2):
                result_list[i].append("__")
        else:
            for i in range(2):
                result_list[i].append(diff_list[i][e])
    return result_list[0], result_list[1], diff_indexes


def _fill_empty_elements(first, second):
    if len(first) != len(second):
        print("WARN:: Frame len is not the same")

        len_first = len(first)
        len_second = len(second)
        if len_first > len_second:
            print("WARN:: First row is longer by the %sB\n" % (len_first - len_second))
            for x in range(len_first - len_second):
                second.append("  ")
        else:
            print("WARN:: Second row is longer by the %sB\n" % (len_second - len_first))
            for x in range(len_second - len_first):
                first.append("  ")


def _prepare(obj):
    def isbytesarray():
        b = obj.split()
        if len(b) == 1:
            return get_hex(obj).split()
        return b
    import sys
    if sys.version_info > (3, 5):
        # TODO very naive hack, but should be ok as temporary fix
        if isinstance(obj, bytes):
            return isbytesarray()
    if hasattr(obj, "hex"):
        return obj.hex().split()
    if not isinstance(obj, str):
        obj = get_hex(obj)
    if isinstance(obj, str):
        obj = isbytesarray()
    return obj


def get_hex(frame, uppercase=False):
    """
    Return a string object of Hex representation of the Scapy's framework
    :param frame: Scapy's Packet Object
    :param uppercase: bool: If True letters be UPPERCASE
    :return: str: Hex   
    """
    if sys.version_info.major == 2:
        import binascii
        str_hex = binascii.b2a_hex(bytes(frame))
    else:
        try:
            str_hex = bytes(frame).hex()
        except TypeError:
            str_hex = bytes(frame, encoding="utf-8").hex()
    j = []
    for e, i in enumerate(str_hex):
        if e % 2:
            j.append(str_hex[e - 1] + str_hex[e])
    if uppercase:
        return ' '.join(j).upper()
    return ' '.join(j).lower()


def show_hex(frame, uppercase=False):
    """Print a hex representation of the Scapy's object"""
    print(get_hex(frame, uppercase=uppercase))


def _create_diff_indexes_list(indexes, max_len):
    new_list = []

    for idx in range(max_len):
        if idx not in indexes:
            new_list.append("..")
            continue

        if idx < 10:
            new_list.append(str("^%s" % idx))
            continue
        new_list.append(str(idx))
    else:
        new_list.append("| position")

    return ' '.join(new_list)


def __process_char(char):
    multiplier = 2
    if not char or not isinstance(char, str):
        return "X" * multiplier
    return char[0] * multiplier


def show_diff(first, second, index=False, extend=False, empty_char="XX"):
    first_row, second_row, indexes_of_diff = diff(first, second)
    first_row_len_bytes = count_bytes(first_row)
    second_row_len_bytes = count_bytes(second_row)

    empty_char = __process_char(empty_char)

    for row in (first_row, second_row):
        for idx, element in enumerate(row):
            if element == "  ":
                row[idx] = empty_char

    print("%s | len: %sB" % (' '.join(first_row), first_row_len_bytes))
    print("%s | len: %sB" % (' '.join(second_row), second_row_len_bytes))
    if index and indexes_of_diff:
        str_bar = "   " * first_row_len_bytes if first_row_len_bytes > second_row_len_bytes else \
            "   " * second_row_len_bytes
        print("%s|\n%s" % (str_bar, _create_diff_indexes_list(indexes_of_diff, max((first_row_len_bytes,
                                                                                    second_row_len_bytes)))))

    if extend:
        more_info = []
        for types in first.class_fieldtype.items():
            for el in types[1].items():
                more_info.append((el[0], el[1].sz))
        print(more_info)

    print("")
    if indexes_of_diff:
        print("Not equal at {}B's".format(len([x for x in first_row if x != "__"])))
        return True
    elif len([x for x in first_row if x != "__"]) == len(first_row):
        print("Not equal")
        return True
    print("Ok")
    return False


def show_diff_full(first, second, index=True, extend=False, empty_char="XX"):
    first_row, second_row, indexes_of_diff = diff(first, second, skip_if_same=False)
    first_row_len_bytes = count_bytes(first_row)
    second_row_len_bytes = count_bytes(second_row)

    empty_char = __process_char(empty_char)

    for row in (first_row, second_row):
        for idx, element in enumerate(row):
            if element == "  ":
                row[idx] = empty_char

    print(' '.join(first_row), "| len: %sB" % first_row_len_bytes)
    print(' '.join(second_row), "| len: %sB" % second_row_len_bytes)
    if index and indexes_of_diff:
        str_bar = "   " * first_row_len_bytes if first_row_len_bytes > second_row_len_bytes else \
            "   " * second_row_len_bytes
        print("%s|\n%s" % (str_bar, _create_diff_indexes_list(indexes_of_diff, max((first_row_len_bytes,
                                                                                    second_row_len_bytes)))))

    if extend:
        more_info = []
        for types in first.class_fieldtype.items():
            for el in types[1].items():
                more_info.append((el[0], el[1].sz))
        print(more_info)

    print("")
    if indexes_of_diff:
        print("Not equal at {}B's".format(len([x for x in first_row if x != "__"])))
        return True
    elif len([x for x in first_row if x != "__"]) == len(first_row):
        print("Not equal")
        return True
    print("Ok")
    return False


def count_bytes(packet_hex_list):
    return len([x for x in packet_hex_list if x != "  "])


def hex_equal(first, second, show_inequalities=True, **kwargs):
    """
    Compare a two hex or hex-able Scapy objects. Return a True if objects are equal
    :param first: str hex or Scapy object
    :param second: str hex or Scapy object
    :param show_inequalities: Print difference if elements are not equal
    :param kwargs: Params for show diff
    :return: bool
    """
    _, _, status = diff(first, second)
    if show_inequalities and status:
        show_diff(first, second, **kwargs)
    # diff returns a list of position on which the object is different
    # if status is a empty Tuple, there's no difference between objects
    if status:
        return False
    return True


def get_diff(first, second):
    """This function is the wrapper for the diff. For backward compatibility"""
    return diff(first, second)


def get_diff_status(first, second):
    _, _, status = diff(first, second)
    return status


@deprecated
def table(first, second):
    f, s, status = diff(first, second)
    show_diff(first, second)
    f_details = first.show(dump=True).split("\n")
    s_details = second.show(dump=True).split("\n")

    for r in range(len(f_details)):
        print("{} {:>40}".format(f_details[r], s_details[r]))

    return status



import unittest

from scapy.layers.inet import IP
from scapy.layers.l2 import Ether

from scapy_helper import get_hex
from scapy_helper.hexdump import hexdump

from scapy.utils import hexdump as scap_hexdump


class TestHexdump(unittest.TestCase):
    def setUp(self):
        self.packet = Ether(dst="ff:ff:ff:ff:ff:ff",
                            src="00:00:00:00:00:00") / IP()
        self.packet_hex = "ff ff ff ff ff ff 00 00 00 00 00 00 08 00 " \
                          "45 00 00 14 00 01 00 00 40 00 fb e8 00 00 " \
                          "00 00 7f 00 00 01"

    def test_hexdump_dump_from_string(self):
        expected_result = '0000ffffffffffff00000000000008004500..............E.' \
                          '00100014000100004000fbe8000000007f00......@.........' \
                          '00200001..'
        result = hexdump(self.packet_hex, dump=True).replace(" ", "").replace("\n", "")
        hexdump(self.packet_hex)
        self.assertEqual(expected_result, result, "String should be the same")

    def test_hexdump_dump_from_string_as_list(self):
        expected_result = ['0000ffffffffffff00000000000008004500..............E.',
                           '00100014000100004000fbe8000000007f00......@.........',
                           '00200001..']
        result = [x.replace(" ", "").replace("\n", "") for x in hexdump(self.packet_hex, dump=True, to_list=True)]

        expected_result = [x.lower() for x in expected_result]
        result = [x.lower() for x in result]

        self.assertIsInstance(result, list, "Dump should be a list")
        self.assertEqual(expected_result, result, "Dump of the hex should be the same")

    def test_hexdump_can_convert_string_on_input(self):
        string = "\x00\x01".encode()
        expected_result = "00000001.."
        result = hexdump(string, dump=True)

        self.assertEqual(result.replace(" ", ""), expected_result, "Dump of hex should be the same")

    def test_hexdump_dump_true(self):
        expected_result = scap_hexdump(self.packet, dump=True).replace(" ", "").replace("\n", "")
        result = hexdump(self.packet, dump=True).replace(" ", "").replace("\n", "")

        self.assertIsInstance(result, str, "Dump should be a string")
        self.assertEqual(expected_result.lower(), result.lower(), "Dump of the hex should be the same")

    def test_hexdump_dump_true_as_list(self):
        expected_result = scap_hexdump(self.packet, dump=True).replace(" ", "").split("\n")
        result = [x.replace(" ", "").replace("\n", "") for x in hexdump(self.packet, dump=True, to_list=True)]

        expected_result = [x.lower() for x in expected_result]
        result = [x.lower() for x in result]

        self.assertIsInstance(result, list, "Dump should be a list")
        self.assertEqual(expected_result, result, "Dump of the hex should be the same")



from unittest import TestCase

from mock import MagicMock
from scapy.layers.l2 import Ether
from scapy.utils import chexdump

from scapy_helper import get_hex, show_diff
from scapy_helper.main import diff, hex_equal, _prepare, get_diff


class TestScapyHelper(TestCase):
    def setUp(self):
        self.ether = Ether(dst="ff:ff:ff:ff:ff:ff",
                           src="00:00:00:00:00:00")
        self.ether2 = Ether(dst="00:00:00:00:00:00",
                            src="00:00:00:00:00:00")

    def test_get_diff_should_be_the_same_as_diff(self):
        fun_diff = diff(self.ether, self.ether2)
        fun_get_diff = get_diff(self.ether, self.ether2)
        self.assertEqual(fun_diff, fun_get_diff, "The result of get_diff and diff should be the same")

    def test_diff_list_equal(self):
        result = diff("ff ff ff ff ff ff 00 00 00 00 00 00 90 00", "ff ff ff ff ff ff 00 00 00 00 00 00 90 00")
        self.assertEqual(
            result,
            (['__', '__', '__', '__', '__', '__', '__', '__', '__', '__', '__', '__', '__', '__'],
             ['__', '__', '__', '__', '__', '__', '__', '__', '__', '__', '__', '__', '__', '__'], []),
            "Results should be equal"
        )

    def test_diff_list_should_be_diff_at_6_position(self):
        result = diff("ff ff ff ff ff ff 11 00 00 00 00 00 90 00", "ff ff ff ff ff ff 00 00 00 00 00 00 90 00")
        self.assertEqual(
            result,
            (['__', '__', '__', '__', '__', '__', '11', '__', '__', '__', '__', '__', '__', '__'],
             ['__', '__', '__', '__', '__', '__', '00', '__', '__', '__', '__', '__', '__', '__'], [6]),
            "Results should be equal"
        )

    def test_diff_refactor(self):
        with self.assertRaises(NotImplementedError):
            diff("00 11", "00 11", "00 00")

        with self.assertRaises(NotImplementedError):
            diff("00 11")

    def test_get_hex(self):
        assert get_hex(self.ether) == "ff ff ff ff ff ff 00 00 00 00 00 00 90 00"

    def test_get_hex_uppercase(self):
        self.assertEqual(
            get_hex(self.ether, uppercase=True),
            "FF FF FF FF FF FF 00 00 00 00 00 00 90 00"
        )

    def test_show_diff(self):
        ether_wrong = "ff ff ff ff ff ff 00 00 00 00 aa 00 90 00"
        first_row, second_row, status = diff(self.ether, ether_wrong)

        self.assertEqual(first_row,
                         "__ __ __ __ __ __ __ __ __ __ 00 __ __ __".split())
        self.assertEqual(second_row,
                         "__ __ __ __ __ __ __ __ __ __ aa __ __ __".split())
        self.assertTrue(status)
        self.assertEqual(status, [10, ])

    def test_show_diff_for_same_elements(self):
        first_row, second_row, status = diff(self.ether, self.ether)

        self.assertEqual(first_row,
                         "__ __ __ __ __ __ __ __ __ __ __ __ __ __".split())
        self.assertEqual(second_row,
                         "__ __ __ __ __ __ __ __ __ __ __ __ __ __".split())
        self.assertFalse(status)

    def test_show_more_diff(self):
        show_diff(self.ether, "ff ff ff ff ff ff 00 00 00 00 00 00 90 00")

    def test_diff_should_return_3_elements(self):
        too_long_ether = get_hex(Ether() / "additional string")

        diff_tuple = diff(self.ether, too_long_ether)
        self.assertEqual(len(diff_tuple), 3, "_diff should return only a 3 elements")

    def test_diff_should_return_same_len_list_if_one_of_them_was_shorter(self):
        too_long_ether = get_hex(Ether() / "additional string")

        diff_tuple = diff(self.ether, too_long_ether)
        self.assertEqual(len(diff_tuple[0]), len(diff_tuple[1]), "List should be the same length")

        self.assertNotEqual(
            len(get_hex(self.ether).split()),
            len(diff_tuple[0])
        )
        self.assertEqual(
            len(too_long_ether.split()),
            len(diff_tuple[1])
        )

    def test_hex_equal(self):
        self.assertTrue(hex_equal(self.ether, "ff ff ff ff ff ff 00 00 00 00 00 00 90 00"))

    def test_hex_equal_with_show_diff_options(self):
        options = {
            "index": True
        }
        self.assertFalse(hex_equal(self.ether, "ff ff ff ff ff ff 0a 00 00 00 00 00 90 00",
                                   show_inequalities=False, **options))

    def test__prepare(self):
        obj = MagicMock()
        obj.hex = MagicMock(return_value="ff ff ff")
        self.assertEqual(_prepare(obj), ["ff", "ff", "ff"], "Object method hex, should return str of hex")

    def test__prepare_encoded(self):
        obj = "\x00\x01".encode()
        self.assertEqual(_prepare(obj), ["00", "01"], "Object method hex, should return str of hex")

    def test_chexdump_native_support(self):
        frame_chexdump = TestScapyHelper.old_method(self.ether)
        self.assertEqual(frame_chexdump, get_hex(self.ether))

    @staticmethod
    def old_method(frame):
        return ' '.join([x.replace("0x", "").replace(",", "") for x in chexdump(frame, dump=True).split()])



from unittest import TestCase

from mock import MagicMock
from scapy.layers.l2 import Ether

from scapy_helper.test_case_extensions.packet_assert import PacketAssert


class TestPacketAssert(TestCase, PacketAssert):
    def test_assert_hex_equal(self):
        self.assertHexEqual(Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:00"),
                            Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:00"))

    def test_assert_hex_equal_w_automatic_conversion(self):
        obj = MagicMock()
        obj.hex = MagicMock(return_value="ff ff ff ff ff ff 00 00 00 00 00 00 90 00")

        self.assertHexEqual(Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:00"),
                            obj)

    def test_assert_hex_not_equal(self):
        self.assertHexNotEqual(Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:00"),
                               Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:aa"))

    def test_negative_assert_hex_equal(self):
        with self.assertRaises(AssertionError):
            self.assertHexEqual(Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:00"),
                                Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:aa"))

    def test_negative_assert_hex_not_equal(self):
        with self.assertRaises(AssertionError):
            self.assertHexNotEqual(Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:00"),
                                   Ether(dst="ff:ff:ff:ff:ff:ff", src="00:00:00:00:00:00"))

    def test_assert_bytes_equal(self):
        with self.assertRaises(AssertionError):
            self.assertBytesEqual(Ether(), Ether(dst="00:00:00:00:00:00"))

    def test_negative_assert_bytes_equal(self):
        self.assertBytesEqual(Ether(), Ether())

    def test_assert_bytes_not_equal(self):
        self.assertBytesNotEqual(Ether(), Ether(dst="00:00:00:00:00:00"))

    def test_negative_bytes_not_equal(self):
        with self.assertRaises(AssertionError):
            self.assertBytesNotEqual(Ether(), Ether())

    def test_assert_hex_different_at_position_zero(self):
        self.assertHexDifferentAt(Ether(dst="00:00:00:00:00:00"),
                                  Ether(dst="01:00:00:00:00:00"),
                                  positions=0,
                                  message="Packets should be different on 0 position")

    def test_negative_assert_hex_different_at_position_two(self):
        with self.assertRaises(AssertionError):
            self.assertHexDifferentAt(Ether(dst="00:00:00:00:00:00"),
                                      Ether(dst="01:00:00:00:00:00"),
                                      positions=2,
                                      message="Packets should be different on 0 position")

    def test_negative_assert_hex_different_at_more_differences(self):
        with self.assertRaises(AssertionError):
            self.assertHexDifferentAt(Ether(dst="00:00:00:00:00:00"),
                                      Ether(dst="01:01:01:00:00:00"),
                                      positions=2,
                                      message="Packets should be different on 0 position")

    def test_assert_hex_different_at_position_list(self):
        self.assertHexDifferentAt(Ether(dst="00:00:00:00:00:00"),
                                  Ether(dst="01:01:00:00:00:00"),
                                  positions=(0, 1),
                                  message="Packets should be different on (0, 1) position")

    def test_negative_assert_hex_different_at_position_wrong_list(self):
        with self.assertRaises(AssertionError):
            self.assertHexDifferentAt(Ether(dst="00:00:00:00:00:00"),
                                      Ether(dst="01:01:00:00:00:00"),
                                      positions=(0, 3),
                                      message="Packets should be different on (0, 1) position")

    def test_negative_assert_hex_different_at_position_wrong_list_half_correct(self):
        with self.assertRaises(AssertionError):
            self.assertHexDifferentAt(Ether(dst="00:00:00:00:00:00"),
                                      Ether(dst="01:01:00:00:00:00"),
                                      positions=(0, 1, 2),
                                      message="Packets should be different on (0, 1) positions")

    def test_negative_assert_hex_different_at_position_hex_not_equal_list_is_subset(self):
        with self.assertRaises(AssertionError):
            self.assertHexDifferentAt(Ether(dst="00:00:00:00:00:00"),
                                      Ether(dst="01:01:01:01:01:01"),
                                      positions=(0, 1, 2),
                                      message="Packets should be different on (0-5) positions")



from unittest import TestCase

from scapy_helper.helpers.ip import int2ip, ip2int


class TestIP(TestCase):
    def setUp(self):
        self.integer = 3232235521
        self.ip = "192.168.0.1"

    def test_ip2int(self):
        self.assertEqual(
            ip2int(self.ip),
            self.integer,
            "Values should be equal"
        )

    def test_int2ip(self):
        self.assertEqual(
            int2ip(self.integer),
            self.ip,
            "Values should be equal"
        )



from unittest import TestCase


class TestInterfaceInteroperability(TestCase):
    def test_version_0_1_5(self):
        try:
            from scapy_helper import show_diff, show_hex, get_hex, table
            from scapy_helper import compare
        except ImportError as e:
            self.fail("Import Error: %s" % e)

    def test_version_0_1_11(self):
        try:
            from scapy_helper import get_hex, show_diff, show_hex, table, hex_equal
            from scapy_helper import Compare
        except ImportError as e:
            self.fail("Import Error: %s" % e)

    def test_version_0_3_0(self):
        try:
            from scapy_helper import get_hex, show_diff, show_hex, table, hex_equal
            from scapy_helper import Compare
            from scapy_helper import PacketAssert
        except ImportError as e:
            self.fail("Import Error: %s" % e)



from tabulate import tabulate

from scapy_helper.main import get_hex, show_diff


class Compare:
    def __init__(self, first, second):
        self.first = first
        self.second = second

    def equal(self):
        """
        Return true if booth elements are equal
        :return: bool
        """
        return not self.diff()

    def hex(self):
        """
        Return tuple with hex elements
        :return: Tuple(str, str)
        """
        return get_hex(self.first), get_hex(self.second)

    def diff(self):
        """
        Show differences between two packets
        :return: bool: Return True if packets are NOT EQUAL
        """
        print("This is temporary -- will be changed in the future")
        return show_diff(self.first, self.second)

    def tdiff(self):
        """[Shortcut] Wrapper for the table_diff"""
        self.table_diff()

    def table_diff(self, index=False):
        """
        Print a difference and print table information about packets
        :param index: Default=False, If True show position under the differ position
        :return: bool: Return True if packets are NOT EQUAL
        """
        def prepare_data(first, second):
            if "=" in first and "=" in second:
                column_a = first.split("=")
                column_b = second.split("=")
                if column_a != column_b:
                    header = "{} != {}".format(column_a[1], column_b[1])
                    return header, column_a[0], column_a[1], column_b[1]
                return None, column_a[0], column_a[1], column_b[1]
            return first, None, None, None

        status = show_diff(self.first, self.second, index=index)
        self._print_table(prepare_data)
        return status

    def _print_table(self, prepare_data):
        """
        Print table base on prepared data
        :param prepare_data:
        :return: None
        """
        f_details = self.first.show(dump=True).split("\n")
        s_details = self.second.show(dump=True).split("\n")
        data = [("Diff or header", "Element", "First", "Second")]
        for r in range(len(f_details)):
            data.append(prepare_data(f_details[r], s_details[r]))
        print(tabulate(data, headers="firstrow", tablefmt="github"))



from scapy_helper.main import _prepare


def chexdump(packet, dump=False, to_list=False):
    """
    Return a chexdump base on packet
    :param packet: String or Scapy object
    :param dump: True if you want to dump instead of print
    :param to_list: True if you want a list of hex instead of a string
    :return: None or Str or List
    """
    def _add_0x_before(p):
        return ["0x%s" % x for x in p]

    processed_packet = _prepare(packet)
    if not processed_packet:
        return

    if dump:
        if to_list:
            return _add_0x_before(processed_packet)
        else:
            return ", ".join(_add_0x_before(processed_packet))
    print(", ".join(_add_0x_before(processed_packet)))





from setuptools import setup, find_packages

with open("./README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="scapy_helper",
    description="Several features that should help you use Scapy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Nex Sabre",
    author_email="nexsabre@protonmail.com",
    version="0.8.0",
    url="https://github.com/NexSabre/scapy_helper",
    packages=find_packages(),
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    license='MIT',
    install_requires=["tabulate~=0.8.7"]
)



import unittest

from scapy.layers.inet import IP, UDP
from scapy.layers.l2 import Ether

from scapy_helper.compare import Compare


class TestCompare(unittest.TestCase):
    def setUp(self):
        self.ether = Ether(dst="ff:ff:ff:ff:ff:ff",
                           src="00:00:00:00:00:00") / IP(src='192.168.1.1',
                                                         dst='192.168.1.20', ttl=15) / UDP()
        self.c_ether = Ether(dst="ff:ff:ff:ff:ff:ff",
                             src="00:00:00:00:00:00") / IP(src='192.168.1.2',
                                                           dst='192.168.1.20', ttl=20) / UDP()

        self.compare = Compare(self.ether, self.c_ether)

    def test_table_diff(self):
        self.compare.table_diff()

    def test_hex(self):
        self.assertEqual(self.compare.hex(),
                         (('ff ff ff ff ff ff 00 00 00 00 00 00 08 00 45 00 00 1c 00 01 00 00 0f 11'
                           ' 28 6b c0 a8 01 01 c0 a8 01 14 00 35 00 35 00 08 7c 0e',
                           'ff ff ff ff ff ff 00 00 00 00 00 00 08 00 45 00 00 1c 00 01 00 00 14 11'
                           ' 23 6a c0 a8 01 02 c0 a8 01 14 00 35 00 35 00 08 7c 0d')))

    def test_diff(self):
        self.assertTrue(self.compare.diff(), "diff() should return a difference")

    def test_equal(self):
        self.assertFalse(self.compare.equal(), "equal() should return false")



import re


def int2mac(integer, upper=False):
    """
    Convert a integer value into a mac address
    :param integer: Integer value of potential mac address
    :param upper: True if you want get address with uppercase
    :return: String mac address in format AA:BB:CC:DD:EE:FF
    """
    def format_int():
        return zip(*[iter('{:012x}'.format(integer))] * 2)

    if type(integer).__name__ not in ("int", "long"):
        # hack: since in the python3, long is not supported
        raise TypeError("Value %s is not a number" % integer)

    mac = ':'.join(['{}{}'.format(a, b)
                    for a, b in format_int()])
    if upper:
        return mac.upper()
    return mac


def mac2int(string):
    """
    Convert a mac address in string format into int value
    :param string: Should be a string if format
    "00:11:22:33:44:55:66"
    :return: Integer value of mac address
    """
    if not isinstance(string, str):
        raise TypeError("Value %s is not a type of string" % string)

    potential_mac = re.match('^((?:(?:[0-9a-f]{2}):){5}[0-9a-f]{2})$',
                             string.lower())
    if not potential_mac:
        raise ValueError("Invalid mac address %s" % string)

    return int(potential_mac
               .group()
               .replace(":", "")
               .replace(".", ""), 16)
